/*
 *  Copyright (C) 2013  Thadeu Lima de Souza Cascardo <cascardo@cascardo.info>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "message.h"
#include "friend.h"
#include "menu.h"
#include <glib.h>
#include <gio/gio.h>

#include <stdio.h>

gboolean ping_timeout(gpointer data)
{
	struct friend *friend = data;
	friend_timeout(friend);
	return G_SOURCE_REMOVE;
}

static GIOChannel *uchannel;
static GSocket *gusock;

static void command(char *buffer, size_t len, GSocketAddress *address)
{
	gchar **args;
	args = g_strsplit(buffer, " ", -1);
	if (args == NULL)
		return;
	menu_run(args, address);
	g_strfreev(args);
}

gboolean message_incoming(GIOChannel *channel, GIOCondition cond, gpointer data)
{
	size_t len;
	char *buffer;
	GSocketAddress *address;
	GInetAddress *iaddress;
	if (!gusock) {
		gusock = g_socket_new_from_fd(g_io_channel_unix_get_fd(channel), NULL);
	}
	len = g_socket_get_available_bytes(gusock);
	if (len <= 0) {
		goto out;
	}
	buffer = g_malloc(len + 1);
	len = g_socket_receive_from(gusock, &address, buffer, len, NULL, NULL);
	buffer[len] = 0;
	iaddress = g_inet_socket_address_get_address(G_INET_SOCKET_ADDRESS(address));
	if (g_inet_address_get_is_loopback(iaddress)) {
		command(buffer, len, address);
	} else {
		struct friend *friend;
		uint16_t port = g_inet_socket_address_get_port(G_INET_SOCKET_ADDRESS(address));
		friend = friend_get_by_address(iaddress, port);
		if (friend) {
			printf("got message from %s\n", friend_get_name(friend));
			friend_got_message(friend, buffer, len);
		} else {
			printf("could not find friend from address %s\n", g_inet_address_to_string(iaddress));
		}
	}
	g_object_unref(address);
	g_free(buffer);
out:
	return TRUE;
}

int message_init(GSocket *sock)
{
	uchannel = g_io_channel_unix_new(g_socket_get_fd(sock));
	g_io_add_watch(uchannel, G_IO_IN, message_incoming, NULL);
	return 0;
}

int ping(struct friend *friend)
{
	int err;
	char ping[5] = "PING";
	err = friend_send_message(friend, ping, 4);
	if (!err)
		g_timeout_add(2000, ping_timeout, friend);
	return err;
}

int pong(struct friend *friend)
{
	char pong[5] = "PONG";
	return friend_send_message(friend, pong, 4);
}
